@@ -25,17 +25,21 @@
  * THE SOFTWARE.
  */
 
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
+#include <libbladeRF.h>
 #include <stdbool.h>
 #include <stdint.h>
-#include <libbladeRF.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
 #include "example_common.h"
 
 /* Just retransmit the received samples */
-bool do_work(int16_t *rx, unsigned int rx_len,
-             bool *have_tx_data, int16_t *tx, unsigned int tx_len)
+bool do_work(int16_t *rx,
+             unsigned int rx_len,
+             bool *have_tx_data,
+             int16_t *tx,
+             unsigned int tx_len)
 {
     static unsigned int call_no = 0;
 
@@ -63,35 +67,26 @@ static int init_sync(struct bladerf *dev)
      * received.
      */
     const unsigned int num_buffers   = 16;
-    const unsigned int buffer_size   = 8192;  /* Must be a multiple of 1024 */
+    const unsigned int buffer_size   = 8192; /* Must be a multiple of 1024 */
     const unsigned int num_transfers = 8;
     const unsigned int timeout_ms    = 3500;
 
-
-    /* Configure both the device's RX and TX modules for use with the synchronous
+    /* Configure both the device's x1 RX and TX channels for use with the
+     * synchronous
      * interface. SC16 Q11 samples *without* metadata are used. */
-    status = bladerf_sync_config(dev,
-                                 BLADERF_MODULE_RX,
-                                 BLADERF_FORMAT_SC16_Q11,
-                                 num_buffers,
-                                 buffer_size,
-                                 num_transfers,
-                                 timeout_ms);
 
+    status = bladerf_sync_config(dev, BLADERF_RX_X1, BLADERF_FORMAT_SC16_Q11,
+                                 num_buffers, buffer_size, num_transfers,
+                                 timeout_ms);
     if (status != 0) {
         fprintf(stderr, "Failed to configure RX sync interface: %s\n",
                 bladerf_strerror(status));
         return status;
     }
 
-    status = bladerf_sync_config(dev,
-                                 BLADERF_MODULE_TX,
-                                 BLADERF_FORMAT_SC16_Q11,
-                                 num_buffers,
-                                 buffer_size,
-                                 num_transfers,
+    status = bladerf_sync_config(dev, BLADERF_TX_X1, BLADERF_FORMAT_SC16_Q11,
+                                 num_buffers, buffer_size, num_transfers,
                                  timeout_ms);
-
     if (status != 0) {
         fprintf(stderr, "Failed to configure TX sync interface: %s\n",
                 bladerf_strerror(status));
@@ -105,26 +100,26 @@ static int init_sync(struct bladerf *dev)
 int sync_rx_example(struct bladerf *dev)
 {
     int status, ret;
-    bool done = false;
+    bool done         = false;
     bool have_tx_data = false;
 
     /** [user_buffers] */
 
     /* "User" samples buffers and their associated sizes, in units of samples.
      * Recall that one sample = two int16_t values. */
-    int16_t *rx_samples = NULL;
-    int16_t *tx_samples = NULL;
+    int16_t *rx_samples            = NULL;
+    int16_t *tx_samples            = NULL;
     const unsigned int samples_len = 10000; /* May be any (reasonable) size */
 
     /* Allocate a buffer to store received samples in */
-    rx_samples = malloc(samples_len * 2 * sizeof(int16_t));
+    rx_samples = malloc(samples_len * 2 * 1 * sizeof(int16_t));
     if (rx_samples == NULL) {
         perror("malloc");
         return BLADERF_ERR_MEM;
     }
 
     /* Allocate a buffer to prepare transmit data in */
-    tx_samples = malloc(samples_len * 2 * sizeof(int16_t));
+    tx_samples = malloc(samples_len * 2 * 1 * sizeof(int16_t));
     if (tx_samples == NULL) {
         perror("malloc");
         free(rx_samples);
@@ -133,7 +128,7 @@ int sync_rx_example(struct bladerf *dev)
 
     /** [user_buffers] */
 
-    /* Initialize synch interface on RX and TX modules */
+    /* Initialize synch interface on RX and TX */
     status = init_sync(dev);
     if (status != 0) {
         goto out;
@@ -141,17 +136,15 @@ int sync_rx_example(struct bladerf *dev)
 
     /** [enable_modules] */
 
-    status = bladerf_enable_module(dev, BLADERF_MODULE_RX, true);
+    status = bladerf_enable_module(dev, BLADERF_RX, true);
     if (status != 0) {
-        fprintf(stderr, "Failed to enable RX module: %s\n",
-                bladerf_strerror(status));
+        fprintf(stderr, "Failed to enable RX: %s\n", bladerf_strerror(status));
         goto out;
     }
 
-    status = bladerf_enable_module(dev, BLADERF_MODULE_TX, true);
+    status = bladerf_enable_module(dev, BLADERF_TX, true);
     if (status != 0) {
-        fprintf(stderr, "Failed to enable RX module: %s\n",
-                bladerf_strerror(status));
+        fprintf(stderr, "Failed to enable TX: %s\n", bladerf_strerror(status));
         goto out;
     }
 
@@ -159,21 +152,18 @@ int sync_rx_example(struct bladerf *dev)
     /** [rxtx_loop] */
 
     while (status == 0 && !done) {
-
         /* Receive samples */
         status = bladerf_sync_rx(dev, rx_samples, samples_len, NULL, 5000);
         if (status == 0) {
-
             /* Process these samples, and potentially produce a response
              * to transmit */
-            done = do_work(rx_samples, samples_len,
-                           &have_tx_data, tx_samples, samples_len);
+            done = do_work(rx_samples, samples_len, &have_tx_data, tx_samples,
+                           samples_len);
 
             if (!done && have_tx_data) {
-
                 /* Transmit a response */
-                status = bladerf_sync_tx(dev, tx_samples, samples_len,
-                                         NULL, 5000);
+                status =
+                    bladerf_sync_tx(dev, tx_samples, samples_len, NULL, 5000);
 
                 if (status != 0) {
                     fprintf(stderr, "Failed to TX samples: %s\n",
@@ -192,25 +182,23 @@ int sync_rx_example(struct bladerf *dev)
         usleep(2000000);
     }
 
-    /** [rxtx_loop] */
+/** [rxtx_loop] */
 
 out:
     ret = status;
 
     /** [disable_modules] */
 
-    /* Disable RX module, shutting down our underlying RX stream */
-    status = bladerf_enable_module(dev, BLADERF_MODULE_RX, false);
+    /* Disable RX, shutting down our underlying RX stream */
+    status = bladerf_enable_module(dev, BLADERF_RX, false);
     if (status != 0) {
-        fprintf(stderr, "Failed to disable RX module: %s\n",
-                bladerf_strerror(status));
+        fprintf(stderr, "Failed to disable RX: %s\n", bladerf_strerror(status));
     }
 
-    /* Disable TX module, shutting down our underlying TX stream */
-    status = bladerf_enable_module(dev, BLADERF_MODULE_TX, false);
+    /* Disable TX, shutting down our underlying TX stream */
+    status = bladerf_enable_module(dev, BLADERF_TX, false);
     if (status != 0) {
-        fprintf(stderr, "Failed to disable TX module: %s\n",
-                bladerf_strerror(status));
+        fprintf(stderr, "Failed to disable TX: %s\n", bladerf_strerror(status));
     }
 
 
@@ -224,16 +212,17 @@ out:
 }
 /** [example_snippet] */
 
-static void usage(const char *argv0) {
+static void usage(const char *argv0)
+{
     printf("Usage: %s [device specifier]\n\n", argv0);
 }
 
 
 int main(int argc, char *argv[])
 {
-    int status = -1;
+    int status          = -1;
     struct bladerf *dev = NULL;
-    const char *devstr = NULL;
+    const char *devstr  = NULL;
 
     if (argc == 2) {
         if (!strcasecmp("-h", argv[1]) || !strcasecmp("--help", argv[1])) {
