@@ -30,15 +30,16 @@ extern "C" {
 #include <errno.h>
 #include "bladeRF.h"    /* Firmware interface */
 
+#include "devinfo.h"
 #include "backend/backend.h"
 #include "backend/usb/usb.h"
-#include "async.h"
+#include "streaming/async.h"
+#include "helpers/timeout.h"
 #include "log.h"
 }
 
 #include <CyAPI.h>
 
-
 /* This GUID must match that in the modified CyUSB3.inf used with the bladeRF */
 static const GUID driver_guid = {
     0x35D5D3F1, 0x9D0E, 0x4F62, 0xBC, 0xFB, 0xB0, 0xD4, 0x8E,0xA6, 0x34, 0x16
@@ -129,6 +130,7 @@ out:
 static inline bool has_bladerf_ids(CCyUSBDevice *dev)
 {
     return ((dev->VendorID == USB_NUAND_VENDOR_ID)          && (dev->ProductID == USB_NUAND_BLADERF_PRODUCT_ID)) ||
+           ((dev->VendorID == USB_NUAND_VENDOR_ID)          && (dev->ProductID == USB_NUAND_BLADERF2_PRODUCT_ID)) ||
            ((dev->VendorID == USB_NUAND_LEGACY_VENDOR_ID)   && (dev->ProductID == USB_NUAND_BLADERF_LEGACY_PRODUCT_ID));
 }
 
@@ -249,6 +251,7 @@ static int open_via_info(void **driver, backend_probe_target probe_target,
 
     if (status != 0) {
         delete dev;
+        ReleaseMutex(cyapi_data->mutex);
         CloseHandle(cyapi_data->mutex);
     }
 
@@ -275,17 +278,33 @@ static int cyapi_change_setting(void *driver, uint8_t setting)
     }
 }
 
-
 static void cyapi_close(void *driver)
 {
     struct bladerf_cyapi *cyapi_data = get_backend_data(driver);
     cyapi_data->dev->Close();
     delete cyapi_data->dev;
+    ReleaseMutex(cyapi_data->mutex);
     CloseHandle(cyapi_data->mutex);
     free(driver);
 
 }
 
+static int cyapi_get_vid_pid(void *driver, uint16_t *vid,
+                             uint16_t *pid)
+{
+    CCyUSBDevice *dev = get_device(driver);
+
+    *vid = dev->VendorID;
+    *pid = dev->ProductID;
+
+    return 0;
+}
+
+static int cyapi_get_flash_id(void *driver, uint8_t *mid, uint8_t *did)
+{
+    return BLADERF_ERR_UNSUPPORTED;
+}
+
 static int cyapi_get_speed(void *driver,
                           bladerf_dev_speed *device_speed)
 {
@@ -507,7 +526,6 @@ out:
     return status;
 }
 
-
 #ifdef LOGGING_ENABLED
 static inline int find_buf(void* ptr, struct bladerf_stream *stream)
 {
@@ -567,7 +585,7 @@ static int submit_transfer(struct bladerf_stream *stream, void *buffer)
 }
 
 static int cyapi_stream(void *driver, struct bladerf_stream *stream,
-                       bladerf_module module)
+                        bladerf_channel_layout layout)
 {
     int status;
     int idx = 0;
@@ -579,25 +597,21 @@ static int cyapi_stream(void *driver, struct bladerf_stream *stream,
     struct bladerf_cyapi *cyapi = get_backend_data(driver);
     struct bladerf_metadata meta;
 
-    assert(stream->dev->transfer_timeout[stream->module] <= ULONG_MAX);
-    if (stream->dev->transfer_timeout[stream->module] == 0) {
+    assert(stream->transfer_timeout <= ULONG_MAX);
+    if (stream->transfer_timeout == 0) {
         timeout_ms = INFINITE;
     } else {
-        timeout_ms = stream->dev->transfer_timeout[stream->module];
+        timeout_ms = stream->transfer_timeout;
     }
 
-    switch (module) {
-        case BLADERF_MODULE_RX:
+    switch (layout & BLADERF_DIRECTION_MASK) {
+        case BLADERF_RX:
             data->ep = get_ep(cyapi->dev, SAMPLE_EP_IN);
             break;
 
-        case BLADERF_MODULE_TX:
+        case BLADERF_TX:
             data->ep = get_ep(cyapi->dev, SAMPLE_EP_OUT);
             break;
-
-        default:
-            assert(!"Invalid module");
-            return BLADERF_ERR_UNEXPECTED;
     }
 
     if (data->ep == NULL) {
@@ -617,7 +631,7 @@ static int cyapi_stream(void *driver, struct bladerf_stream *stream,
     MUTEX_LOCK(&stream->lock);
 
     for (unsigned int i = 0; i < data->num_transfers && status == 0; i++) {
-        if (module == BLADERF_MODULE_TX) {
+        if ((layout & BLADERF_DIRECTION_MASK) == BLADERF_TX) {
             next_buffer = stream->cb(stream->dev, stream, &meta, NULL,
                                      stream->samples_per_buffer,
                                      stream->user_data);
@@ -726,6 +740,7 @@ out:
 
     return 0;
 }
+
 /* The top-level code will have acquired the stream->lock for us */
 int cyapi_submit_stream_buffer(void *driver, struct bladerf_stream *stream,
                                void *buffer, unsigned int timeout_ms,
@@ -798,6 +813,8 @@ extern "C" {
         FIELD_INIT(.probe, cyapi_probe),
         FIELD_INIT(.open, cyapi_open),
         FIELD_INIT(.close, cyapi_close),
+        FIELD_INIT(.get_vid_pid, cyapi_get_vid_pid),
+        FIELD_INIT(.get_flash_id, cyapi_get_flash_id),
         FIELD_INIT(.get_speed, cyapi_get_speed),
         FIELD_INIT(.change_setting, cyapi_change_setting),
         FIELD_INIT(.control_transfer, cyapi_control_transfer),
@@ -812,7 +829,7 @@ extern "C" {
     };
 
     struct usb_driver usb_driver_cypress = {
+        FIELD_INIT(.fn, &cypress_fns),
         FIELD_INIT(.id, BLADERF_BACKEND_CYPRESS),
-        FIELD_INIT(.fn, &cypress_fns)
     };
 }
