@@ -18,16 +18,28 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
+
+#include "host_config.h"
 #include <stdlib.h>
+#include <string.h>
 #include <pthread.h>
 #include <errno.h>
 #include <libusb.h>
-#include "bladeRF.h"    /* Firmware interface */
+#if 1 == BLADERF_OS_FREEBSD  
+#include <limits.h>
+#endif // BLADERF_OS_FREEBSD
 
+#include "log.h"
+
+#include "devinfo.h"
 #include "backend/backend.h"
 #include "backend/usb/usb.h"
-#include "async.h"
-#include "log.h"
+#include "streaming/async.h"
+#include "helpers/timeout.h"
+
+#include "bladeRF.h"
+
+#include "host_config.h"
 
 #ifndef LIBUSB_HANDLE_EVENTS_TIMEOUT_NSEC
 #   define LIBUSB_HANDLE_EVENTS_TIMEOUT_NSEC    (15 * 1000)
@@ -37,6 +49,9 @@ struct bladerf_lusb {
     libusb_device           *dev;
     libusb_device_handle    *handle;
     libusb_context          *context;
+#if 1 == BLADERF_OS_WINDOWS
+    HANDLE                  mutex;
+#endif // BLADERF_OS_WINDOWS
 };
 
 typedef enum {
@@ -64,8 +79,8 @@ static inline struct bladerf_lusb * lusb_backend(struct bladerf *dev)
 {
     struct bladerf_usb *usb;
 
-    assert(dev && dev->backend);
-    usb = (struct bladerf_usb *) dev->backend;
+    assert(dev && dev->backend_data);
+    usb = (struct bladerf_usb *) dev->backend_data;
 
     assert(usb->driver);
     return (struct bladerf_lusb *) usb->driver;
@@ -128,33 +143,56 @@ static int get_devinfo(libusb_device *dev, struct bladerf_devinfo *info)
     libusb_device_handle *handle;
     struct libusb_device_descriptor desc;
 
+    /* Populate device info */
+    bladerf_init_devinfo(info);
+    info->backend  = BLADERF_BACKEND_LIBUSB;
+    info->usb_bus  = libusb_get_bus_number(dev);
+    info->usb_addr = libusb_get_device_address(dev);
+
     status = libusb_open(dev, &handle);
 
     if (status == 0) {
-        /* Populate device info */
-        info->backend = BLADERF_BACKEND_LIBUSB;
-        info->usb_bus = libusb_get_bus_number(dev);
-        info->usb_addr = libusb_get_device_address(dev);
-
         status = libusb_get_device_descriptor(dev, &desc);
         if (status != 0) {
             memset(info->serial, 0, BLADERF_SERIAL_LENGTH);
         } else {
             status = libusb_get_string_descriptor_ascii(
-                                        handle, desc.iSerialNumber,
-                                        (unsigned char *)&info->serial,
-                                        BLADERF_SERIAL_LENGTH);
+                handle, desc.iSerialNumber, (unsigned char *)&info->serial,
+                BLADERF_SERIAL_LENGTH);
 
             /* Consider this to be non-fatal, otherwise firmware <= 1.1
              * wouldn't be able to get far enough to upgrade */
             if (status < 0) {
                 log_debug("Failed to retrieve serial number\n");
                 memset(info->serial, 0, BLADERF_SERIAL_LENGTH);
-            } else {
-                /* Adjust for > 0 return code */
-                status = 0;
             }
 
+            status = libusb_get_string_descriptor_ascii(
+                handle, desc.iManufacturer,
+                (unsigned char *)&info->manufacturer,
+                BLADERF_DESCRIPTION_LENGTH);
+
+            if (status < 0) {
+                log_debug("Failed to retrieve manufacturer string\n");
+                memset(info->manufacturer, 0, BLADERF_DESCRIPTION_LENGTH);
+            }
+
+            status = libusb_get_string_descriptor_ascii(
+                handle, desc.iProduct, (unsigned char *)&info->product,
+                BLADERF_DESCRIPTION_LENGTH);
+
+            if (status < 0) {
+                log_debug("Failed to retrieve product string\n");
+                memset(info->product, 0, BLADERF_DESCRIPTION_LENGTH);
+            }
+
+            log_debug("Bus %03d Device %03d: %s %s, serial %s\n", info->usb_bus,
+                      info->usb_addr, info->manufacturer, info->product,
+                      info->serial);
+
+            if (status > 0) {
+                status = 0;
+            }
         }
 
         libusb_close(handle);
@@ -190,6 +228,7 @@ static bool device_is_fx3_bootloader(libusb_device *dev)
 static inline bool device_has_bladeRF_ids(libusb_device *dev)
 {
     return device_has_vid_pid(dev, USB_NUAND_VENDOR_ID, USB_NUAND_BLADERF_PRODUCT_ID) ||
+           device_has_vid_pid(dev, USB_NUAND_VENDOR_ID, USB_NUAND_BLADERF2_PRODUCT_ID) ||
            device_has_vid_pid(dev, USB_NUAND_LEGACY_VENDOR_ID, USB_NUAND_BLADERF_LEGACY_PRODUCT_ID);
 
 }
@@ -281,30 +320,45 @@ static int lusb_probe(backend_probe_target probe_target,
     /* Iterate through all the USB devices */
     for (i = 0, n = 0; i < count && status == 0; i++) {
         if (device_is_probe_target(probe_target, list[i])) {
+            bool do_add = true;
 
             /* Open the USB device and get some information */
             status = get_devinfo(list[i], &info);
             if (status) {
-                /* We may not be able to open the device if another
-                 * driver (e.g., CyUSB3) is associated with it. Therefore,
-                 * just log to the debug level and carry on. */
+                /* We may not be able to open the device if another driver
+                 * (e.g., CyUSB3) is associated with it. Therefore, just log to
+                 * the debug level and carry on. */
                 log_debug("Could not open device: %s\n",
-                          libusb_error_name(status) );
+                          libusb_error_name(status));
 
-                if (status == LIBUSB_ERROR_ACCESS && !printed_access_warning) {
-                    printed_access_warning = true;
-                    log_warning("Found a bladeRF via VID/PID, but could not "
-                                "open it due to insufficient permissions.\n");
+                if (status == LIBUSB_ERROR_ACCESS) {
+                    /* If it's an access error, odds are good this is happening
+                     * because we've already got the device open. Pass the info
+                     * we have back to the caller. */
+                    do_add = true;
+
+                    if (!printed_access_warning) {
+                        printed_access_warning = true;
+                        log_warning(
+                            "Found a bladeRF via VID/PID, but could not open "
+                            "it due to insufficient permissions, or because "
+                            "the device is already open.\n");
+                    }
+                } else {
+                    do_add = false;
                 }
 
                 /* Don't stop probing because one device was "problematic" */
                 status = 0;
-            } else {
+            }
+
+            if (do_add) {
                 info.instance = n++;
+
                 status = bladerf_devinfo_list_add(info_list, &info);
-                if( status ) {
+                if (status) {
                     log_error("Could not add device to list: %s\n",
-                              bladerf_strerror(status) );
+                              bladerf_strerror(status));
                 } else {
                     log_verbose("Added instance %d to device list\n",
                                 info.instance);
@@ -336,6 +390,40 @@ static void get_libusb_version(char *buf, size_t buf_len)
 }
 #endif
 
+static int create_device_mutex(const struct bladerf_devinfo *info,
+                               struct bladerf_lusb *dev)
+{
+    int status = 0;
+
+#if 1 == BLADERF_OS_WINDOWS
+    const char prefix[] = "Global\\bladeRF-";
+    const size_t mutex_name_len = strlen(prefix) + BLADERF_SERIAL_LENGTH;
+    char *mutex_name = (char *)calloc(mutex_name_len, 1);
+    DWORD last_error;
+
+    if (NULL == mutex_name) {
+        return BLADERF_ERR_MEM;
+    }
+
+    snprintf(mutex_name, mutex_name_len, "%s%s", prefix, info->serial);
+    log_verbose("Mutex name: %s\n", mutex_name);
+
+    SetLastError(0);
+    dev->mutex = CreateMutex(NULL, true, mutex_name);
+    last_error = GetLastError();
+    if (NULL == dev->mutex || last_error != 0) {
+        log_debug("Unable to create device mutex: mutex=%p, last_error=%ld\n",
+                  dev->mutex, last_error);
+        status = BLADERF_ERR_NODEV;
+        ReleaseMutex(dev->mutex);
+        CloseHandle(dev->mutex);
+    }
+
+    free(mutex_name);
+#endif // BLADERF_OS_WINDOWS
+
+    return status;
+}
 
 static int open_device(const struct bladerf_devinfo *info,
                        libusb_context *context,
@@ -349,7 +437,7 @@ static int open_device(const struct bladerf_devinfo *info,
 
     dev = (struct bladerf_lusb *) calloc(1, sizeof(dev[0]));
     if (dev == NULL) {
-        log_debug("Failed allocate handle for instance %d.\n",
+        log_debug("Failed to allocate handle for instance %d.\n",
                   info->instance);
 
         /* Report "no device" so we could try again with
@@ -366,7 +454,7 @@ static int open_device(const struct bladerf_devinfo *info,
                   info->instance, libusb_error_name(status));
 
         status = error_conv(status);
-        goto error;
+        goto out;
     }
 
     status = libusb_claim_interface(dev->handle, 0);
@@ -375,10 +463,19 @@ static int open_device(const struct bladerf_devinfo *info,
                   info->instance, libusb_error_name(status));
 
         status = error_conv(status);
-        goto error;
+        goto out;
     }
 
-error:
+    status = create_device_mutex(info, dev);
+    if (status < 0) {
+        log_debug("Failed to get device mutex for instance %d: %s\n",
+                  info->instance, bladerf_strerror(status));
+
+        status = error_conv(status);
+        goto out;
+    }
+
+out:
     if (status != 0) {
         if (dev->handle != NULL) {
             libusb_close(dev->handle);
@@ -497,6 +594,11 @@ static int reset_and_reopen(libusb_context *context,
 
         libusb_release_interface((*dev)->handle, 0);
         libusb_close((*dev)->handle);
+#if 1 == BLADERF_OS_WINDOWS
+        ReleaseMutex((*dev)->mutex);
+        CloseHandle((*dev)->mutex);
+#endif // BLADERF_OS_WINDOWS
+
         *dev = NULL;
 
         memcpy(&new_info, info, sizeof(new_info));
@@ -515,7 +617,6 @@ static int reset_and_reopen(libusb_context *context,
 }
 #endif
 
-
 static int lusb_open(void **driver,
                      struct bladerf_devinfo *info_in,
                      struct bladerf_devinfo *info_out)
@@ -583,7 +684,6 @@ static int lusb_change_setting(void *driver, uint8_t setting)
     return error_conv(status);
 }
 
-
 static void lusb_close(void *driver)
 {
     int status;
@@ -597,6 +697,10 @@ static void lusb_close(void *driver)
 
     libusb_close(lusb->handle);
     libusb_exit(lusb->context);
+#if 1 == BLADERF_OS_WINDOWS
+    ReleaseMutex(lusb->mutex);
+    CloseHandle(lusb->mutex);
+#endif // BLADERF_OS_WINDOWS
     free(lusb);
 }
 
@@ -620,6 +724,11 @@ static void lusb_close_bootloader(void *driver)
             libusb_exit(lusb->context);
         }
 
+#if 1 == BLADERF_OS_WINDOWS
+        ReleaseMutex(lusb->mutex);
+        CloseHandle(lusb->mutex);
+#endif // BLADERF_OS_WINDOWS
+
         free(lusb);
     }
 }
@@ -709,6 +818,26 @@ error:
     return status;
 }
 
+static int lusb_get_vid_pid(void *driver, uint16_t *vid,
+                            uint16_t *pid)
+{
+    struct bladerf_lusb *lusb = (struct bladerf_lusb *)driver;
+    struct libusb_device_descriptor desc;
+    int status;
+
+    status = libusb_get_device_descriptor(lusb->dev, &desc);
+    if (status != 0) {
+        log_debug("Couldn't get device descriptor: %s\n",
+                  libusb_error_name(status));
+         return BLADERF_ERR_IO;
+    }
+
+    *vid = desc.idVendor;
+    *pid = desc.idProduct;
+
+    return 0;
+}
+
 static int lusb_get_speed(void *driver,
                           bladerf_dev_speed *device_speed)
 {
@@ -829,7 +958,7 @@ static int lusb_bulk_transfer(void *driver, uint8_t endpoint, void *buffer,
 
     status = error_conv(status);
     if (status == 0 && ((uint32_t)n_transferred != buffer_len)) {
-        log_debug("Short bulk transfer: requeted=%u, transferred=%u\n",
+        log_debug("Short bulk transfer: requested=%u, transferred=%u\n",
                   buffer_len, n_transferred);
         status = BLADERF_ERR_IO;
     }
@@ -869,10 +998,9 @@ static inline void cancel_all_transfers(struct bladerf_stream *stream)
 
     for (i = 0; i < stream_data->num_transfers; i++) {
         if (stream_data->transfer_status[i] == TRANSFER_IN_FLIGHT) {
-
             status = libusb_cancel_transfer(stream_data->transfers[i]);
             if (status < 0 && status != LIBUSB_ERROR_NOT_FOUND) {
-                log_error("Error canceling transfer (%d): %s\r\n",
+                log_error("Error canceling transfer (%d): %s\n",
                         status, libusb_error_name(status));
             } else {
                 stream_data->transfer_status[i] = TRANSFER_CANCEL_PENDING;
@@ -899,7 +1027,7 @@ static int submit_transfer(struct bladerf_stream *stream, void *buffer);
 static void LIBUSB_CALL lusb_stream_cb(struct libusb_transfer *transfer)
 {
     struct bladerf_stream *stream = transfer->user_data;
-    void *next_buffer = NULL;
+    void *next_buffer             = NULL;
     struct bladerf_metadata metadata;
     struct lusb_stream_data *stream_data = stream->backend_data;
     size_t transfer_i;
@@ -914,7 +1042,7 @@ static void LIBUSB_CALL lusb_stream_cb(struct libusb_transfer *transfer)
            stream_data->transfer_status[transfer_i] == TRANSFER_CANCEL_PENDING);
 
     if (transfer_i >= stream_data->num_transfers) {
-        log_error("Unable to find transfer");
+        log_error("Unable to find transfer\n");
         stream->state = STREAM_SHUTTING_DOWN;
     } else {
         stream_data->transfer_status[transfer_i] = TRANSFER_AVAIL;
@@ -924,11 +1052,10 @@ static void LIBUSB_CALL lusb_stream_cb(struct libusb_transfer *transfer)
 
     /* Check to see if the transfer has been cancelled or errored */
     if (transfer->status != LIBUSB_TRANSFER_COMPLETED) {
-
         /* Errored out for some reason .. */
         stream->state = STREAM_SHUTTING_DOWN;
 
-        switch(transfer->status) {
+        switch (transfer->status) {
             case LIBUSB_TRANSFER_CANCELLED:
                 /* We expect this case when we begin tearing down the stream */
                 break;
@@ -944,14 +1071,16 @@ static void LIBUSB_CALL lusb_stream_cb(struct libusb_transfer *transfer)
                 stream->error_code = BLADERF_ERR_IO;
                 break;
 
-            case LIBUSB_TRANSFER_OVERFLOW :
+            case LIBUSB_TRANSFER_OVERFLOW:
                 log_error("Got transfer over for buffer %p, "
-                            "transfer \"actual_length\" = %d\n",
-                            transfer->buffer, transfer->actual_length);
+                          "transfer \"actual_length\" = %d\n",
+                          transfer->buffer, transfer->actual_length);
                 stream->error_code = BLADERF_ERR_IO;
                 break;
 
             case LIBUSB_TRANSFER_TIMED_OUT:
+                log_error("Transfer timed out for buffer %p\n",
+                          transfer->buffer);
                 stream->error_code = BLADERF_ERR_TIMEOUT;
                 break;
 
@@ -960,27 +1089,21 @@ static void LIBUSB_CALL lusb_stream_cb(struct libusb_transfer *transfer)
                 break;
 
             default:
-                log_error( "Unexpected transfer status: %d\n", transfer->status );
+                log_error("Unexpected transfer status: %d\n", transfer->status);
                 break;
         }
-
     }
 
     if (stream->state == STREAM_RUNNING) {
-
         /* Sanity check for debugging purposes */
         if (transfer->length != transfer->actual_length) {
-            log_warning( "Received short transfer\n" );
+            log_warning("Received short transfer\n");
         }
 
-       /* Call user callback requesting more data to transmit */
+        /* Call user callback requesting more data to transmit */
         next_buffer = stream->cb(
-                        stream->dev,
-                        stream,
-                        &metadata,
-                        transfer->buffer,
-                        bytes_to_sc16q11(transfer->actual_length),
-                        stream->user_data);
+            stream->dev, stream, &metadata, transfer->buffer,
+            bytes_to_sc16q11(transfer->actual_length), stream->user_data);
 
         if (next_buffer == BLADERF_STREAM_SHUTDOWN) {
             stream->state = STREAM_SHUTTING_DOWN;
@@ -998,7 +1121,6 @@ static void LIBUSB_CALL lusb_stream_cb(struct libusb_transfer *transfer)
     /* Check to see if all the transfers have been cancelled,
      * and if so, clean up the stream */
     if (stream->state == STREAM_SHUTTING_DOWN) {
-
         /* We know we're done when all of our transfers have returned to their
          * "available" states */
         if (stream_data->num_avail == stream_data->num_transfers) {
@@ -1024,7 +1146,7 @@ get_next_available_transfer(struct lusb_stream_data *stream_data)
                 stream_data->out_of_order_event == false) {
 
                 log_warning("Transfer callback occurred out of order. "
-                            "(Warning only this time.)\r\n");
+                            "(Warning only this time.)\n");
                 stream_data->out_of_order_event = true;
             }
 
@@ -1048,7 +1170,7 @@ static int submit_transfer(struct bladerf_stream *stream, void *buffer)
     const size_t bytes_per_buffer = async_stream_buf_bytes(stream);
     size_t prev_idx;
     const unsigned char ep =
-        stream->module == BLADERF_MODULE_TX ? SAMPLE_EP_OUT : SAMPLE_EP_IN;
+        (stream->layout & BLADERF_DIRECTION_MASK) == BLADERF_TX ? SAMPLE_EP_OUT : SAMPLE_EP_IN;
 
     transfer = get_next_available_transfer(stream_data);
     assert(transfer != NULL);
@@ -1061,7 +1183,7 @@ static int submit_transfer(struct bladerf_stream *stream, void *buffer)
                               (int)bytes_per_buffer,
                               lusb_stream_cb,
                               stream,
-                              stream->dev->transfer_timeout[stream->module]);
+                              stream->transfer_timeout);
 
     prev_idx = stream_data->i;
     stream_data->transfer_status[stream_data->i] = TRANSFER_IN_FLIGHT;
@@ -1103,7 +1225,6 @@ static int submit_transfer(struct bladerf_stream *stream, void *buffer)
     return error_conv(status);
 }
 
-
 static int lusb_init_stream(void *driver, struct bladerf_stream *stream,
                             size_t num_transfers)
 {
@@ -1183,7 +1304,7 @@ error:
 }
 
 static int lusb_stream(void *driver, struct bladerf_stream *stream,
-                       bladerf_module module)
+                       bladerf_channel_layout layout)
 {
     size_t i;
     int status = 0;
@@ -1201,7 +1322,7 @@ static int lusb_stream(void *driver, struct bladerf_stream *stream,
 
     /* Set up initial set of buffers */
     for (i = 0; i < stream_data->num_transfers; i++) {
-        if (module == BLADERF_MODULE_TX) {
+        if ((layout & BLADERF_DIRECTION_MASK) == BLADERF_TX) {
             buffer = stream->cb(dev,
                                 stream,
                                 &metadata,
@@ -1255,6 +1376,7 @@ static int lusb_stream(void *driver, struct bladerf_stream *stream,
 
     return status;
 }
+
 /* The top-level code will have aquired the stream->lock for us */
 int lusb_submit_stream_buffer(void *driver, struct bladerf_stream *stream,
                               void *buffer, unsigned int timeout_ms,
@@ -1277,7 +1399,7 @@ int lusb_submit_stream_buffer(void *driver, struct bladerf_stream *stream,
     if (stream_data->num_avail == 0) {
         if (nonblock) {
             log_debug("Non-blocking buffer submission requested, but no "
-                      "transfers are currently available.");
+                      "transfers are currently available.\n");
 
             return BLADERF_ERR_WOULD_BLOCK;
         }
@@ -1335,6 +1457,8 @@ static const struct usb_fns libusb_fns = {
     FIELD_INIT(.probe, lusb_probe),
     FIELD_INIT(.open, lusb_open),
     FIELD_INIT(.close, lusb_close),
+    FIELD_INIT(.get_vid_pid, lusb_get_vid_pid),
+    FIELD_INIT(.get_flash_id, NULL),
     FIELD_INIT(.get_speed, lusb_get_speed),
     FIELD_INIT(.change_setting, lusb_change_setting),
     FIELD_INIT(.control_transfer, lusb_control_transfer),
@@ -1349,6 +1473,6 @@ static const struct usb_fns libusb_fns = {
 };
 
 const struct usb_driver usb_driver_libusb = {
+    FIELD_INIT(.fn, &libusb_fns),
     FIELD_INIT(.id, BLADERF_BACKEND_LIBUSB),
-    FIELD_INIT(.fn, &libusb_fns)
 };
