@@ -25,15 +25,17 @@
 #include <stdbool.h>
 #include <inttypes.h>
 
-#include "bladerf_priv.h"
+#include "log.h"
+#include "conversions.h"
+
 #include "usb.h"
 #include "nios_access.h"
 #include "nios_pkt_formats.h"
-#include "capabilities.h"
-#include "log.h"
 
-//#define PRINT_BUFFERS
-#ifdef PRINT_BUFFERS
+#include "board/board.h"
+#include "helpers/version.h"
+
+#if 0
 static void print_buf(const char *msg, const uint8_t *buf, size_t len)
 {
     size_t i;
@@ -42,13 +44,7 @@ static void print_buf(const char *msg, const uint8_t *buf, size_t len)
     }
 
     for (i = 0; i < len; i++) {
-        if (i == 0) {
-            fprintf(stderr, "  0x%02x,", buf[i]);
-        } else if ((i + 1) % 8 == 0) {
-            fprintf(stderr, " 0x%02x,\n ", buf[i]);
-        } else {
-            fprintf(stderr, " 0x%02x,", buf[i]);
-        }
+        fprintf(stderr, " %02x", buf[i]);
     }
 
     fputc('\n', stderr);
@@ -60,38 +56,59 @@ static void print_buf(const char *msg, const uint8_t *buf, size_t len)
 /* Buf is assumed to be NIOS_PKT_LEN bytes */
 static int nios_access(struct bladerf *dev, uint8_t *buf)
 {
+    struct bladerf_usb *usb = dev->backend_data;
     int status;
-    void *driver;
-    struct bladerf_usb *usb = usb_backend(dev, &driver);
 
-    print_buf("NIOS II request:\n", buf, NIOS_PKT_LEN);
+    print_buf("NIOS II REQ:", buf, NIOS_PKT_LEN);
 
     /* Send the command */
-    status = usb->fn->bulk_transfer(driver, PERIPHERAL_EP_OUT,
-                                     buf, NIOS_PKT_LEN,
-                                     PERIPHERAL_TIMEOUT_MS);
+    status = usb->fn->bulk_transfer(usb->driver, PERIPHERAL_EP_OUT, buf,
+                                    NIOS_PKT_LEN, PERIPHERAL_TIMEOUT_MS);
     if (status != 0) {
-        log_debug("Failed to send NIOS II request: %s\n",
+        log_error("Failed to send NIOS II request: %s\n",
                   bladerf_strerror(status));
         return status;
     }
 
     /* Retrieve the request */
-    status = usb->fn->bulk_transfer(driver, PERIPHERAL_EP_IN,
-                                    buf, NIOS_PKT_LEN,
-                                    PERIPHERAL_TIMEOUT_MS);
-
+    status = usb->fn->bulk_transfer(usb->driver, PERIPHERAL_EP_IN, buf,
+                                    NIOS_PKT_LEN, PERIPHERAL_TIMEOUT_MS);
     if (status != 0) {
-        log_debug("Failed to receive NIOS II response: %s\n",
+        log_error("Failed to receive NIOS II response: %s\n",
                   bladerf_strerror(status));
     }
 
-    print_buf("NIOS II response:\n", buf, NIOS_PKT_LEN);
+    print_buf("NIOS II res:", buf, NIOS_PKT_LEN);
 
     return status;
 }
 
-int nios_8x8_read(struct bladerf *dev, uint8_t id, uint8_t addr, uint8_t *data)
+/* Variant that doesn't output to log_error on error. */
+static int nios_access_quiet(struct bladerf *dev, uint8_t *buf)
+{
+    struct bladerf_usb *usb = dev->backend_data;
+    int status;
+
+    print_buf("NIOS II REQ:", buf, NIOS_PKT_LEN);
+
+    /* Send the command */
+    status = usb->fn->bulk_transfer(usb->driver, PERIPHERAL_EP_OUT, buf,
+                                    NIOS_PKT_LEN, PERIPHERAL_TIMEOUT_MS);
+    if (status != 0) {
+        return status;
+    }
+
+    /* Retrieve the request */
+    status = usb->fn->bulk_transfer(usb->driver, PERIPHERAL_EP_IN, buf,
+                                    NIOS_PKT_LEN, PERIPHERAL_TIMEOUT_MS);
+
+    print_buf("NIOS II res:", buf, NIOS_PKT_LEN);
+
+    return status;
+}
+
+static int nios_8x8_read(struct bladerf *dev, uint8_t id,
+                         uint8_t addr, uint8_t *data)
 {
     int status;
     uint8_t buf[NIOS_PKT_LEN];
@@ -115,7 +132,8 @@ int nios_8x8_read(struct bladerf *dev, uint8_t id, uint8_t addr, uint8_t *data)
     }
 }
 
-int nios_8x8_write(struct bladerf *dev, uint8_t id, uint8_t addr, uint8_t data)
+static int nios_8x8_write(struct bladerf *dev, uint8_t id,
+                          uint8_t addr, uint8_t data)
 {
     int status;
     uint8_t buf[NIOS_PKT_LEN];
@@ -138,8 +156,8 @@ int nios_8x8_write(struct bladerf *dev, uint8_t id, uint8_t addr, uint8_t data)
     }
 }
 
-int nios_8x16_read(struct bladerf *dev, uint8_t id,
-                   uint8_t addr, uint16_t *data)
+static int nios_8x16_read(struct bladerf *dev, uint8_t id,
+                          uint8_t addr, uint16_t *data)
 {
     int status;
     uint8_t buf[NIOS_PKT_LEN];
@@ -165,8 +183,8 @@ int nios_8x16_read(struct bladerf *dev, uint8_t id,
     }
 }
 
-int nios_8x16_write(struct bladerf *dev, uint8_t id,
-                    uint8_t addr, uint16_t data)
+static int nios_8x16_write(struct bladerf *dev, uint8_t id,
+                           uint8_t addr, uint16_t data)
 {
     int status;
     uint8_t buf[NIOS_PKT_LEN];
@@ -189,8 +207,8 @@ int nios_8x16_write(struct bladerf *dev, uint8_t id,
     }
 }
 
-int nios_8x32_read(struct bladerf *dev, uint8_t id,
-                   uint8_t addr, uint32_t *data)
+static int nios_8x32_read(struct bladerf *dev, uint8_t id,
+                          uint8_t addr, uint32_t *data)
 {
     int status;
     uint8_t buf[NIOS_PKT_LEN];
@@ -214,8 +232,8 @@ int nios_8x32_read(struct bladerf *dev, uint8_t id,
     }
 }
 
-int nios_8x32_write(struct bladerf *dev, uint8_t id,
-                    uint8_t addr, uint32_t data)
+static int nios_8x32_write(struct bladerf *dev, uint8_t id,
+                           uint8_t addr, uint32_t data)
 {
     int status;
     uint8_t buf[NIOS_PKT_LEN];
@@ -238,6 +256,120 @@ int nios_8x32_write(struct bladerf *dev, uint8_t id,
     }
 }
 
+static int nios_16x64_read(struct bladerf *dev,
+                           uint8_t id,
+                           uint16_t addr,
+                           uint64_t *data)
+{
+    int status;
+    uint8_t buf[NIOS_PKT_LEN];
+    bool success;
+
+    nios_pkt_16x64_pack(buf, id, false, addr, 0);
+
+    /* RFIC access times out occasionally, and this is fine. */
+    if (NIOS_PKT_16x64_TARGET_RFIC == id) {
+        status = nios_access_quiet(dev, buf);
+    } else {
+        status = nios_access(dev, buf);
+    }
+
+    if (status != 0) {
+        return status;
+    }
+
+    nios_pkt_16x64_resp_unpack(buf, NULL, NULL, NULL, data, &success);
+
+    if (success) {
+        return 0;
+    } else {
+        *data = 0;
+        log_debug("%s: response packet reported failure.\n", __FUNCTION__);
+        return BLADERF_ERR_FPGA_OP;
+    }
+}
+
+static int nios_16x64_write(struct bladerf *dev,
+                            uint8_t id,
+                            uint16_t addr,
+                            uint64_t data)
+{
+    int status;
+    uint8_t buf[NIOS_PKT_LEN];
+    bool success;
+
+    nios_pkt_16x64_pack(buf, id, true, addr, data);
+
+    /* RFIC access times out occasionally, and this is fine. */
+    if (NIOS_PKT_16x64_TARGET_RFIC == id) {
+        status = nios_access_quiet(dev, buf);
+    } else {
+        status = nios_access(dev, buf);
+    }
+
+    if (status != 0) {
+        return status;
+    }
+
+    nios_pkt_16x64_resp_unpack(buf, NULL, NULL, NULL, NULL, &success);
+
+    if (success) {
+        return 0;
+    } else {
+        log_debug("%s: response packet reported failure.\n", __FUNCTION__);
+        return BLADERF_ERR_FPGA_OP;
+    }
+}
+
+static int nios_32x32_read(struct bladerf *dev, uint32_t id,
+                          uint32_t addr, uint32_t *data)
+{
+    int status;
+    uint8_t buf[NIOS_PKT_LEN];
+    bool success;
+
+    nios_pkt_32x32_pack(buf, id, false, addr, 0);
+
+    status = nios_access(dev, buf);
+    if (status != 0) {
+        return status;
+    }
+
+    nios_pkt_32x32_resp_unpack(buf, NULL, NULL, NULL, data, &success);
+
+    if (success) {
+        return 0;
+    } else {
+        *data = 0;
+        log_debug("%s: response packet reported failure.\n", __FUNCTION__);
+        return BLADERF_ERR_FPGA_OP;
+    }
+}
+
+static int nios_32x32_write(struct bladerf *dev, uint32_t id,
+                           uint32_t addr, uint32_t data)
+{
+    int status;
+    uint8_t buf[NIOS_PKT_LEN];
+    bool success;
+
+    nios_pkt_32x32_pack(buf, id, true, addr, data);
+
+    status = nios_access(dev, buf);
+    if (status != 0) {
+        return status;
+    }
+
+    nios_pkt_32x32_resp_unpack(buf, NULL, NULL, NULL, NULL, &success);
+
+    if (success) {
+        return 0;
+    } else {
+        log_debug("%s: response packet reported failure.\n", __FUNCTION__);
+        return BLADERF_ERR_FPGA_OP;
+    }
+}
+
 static int nios_32x32_masked_read(struct bladerf *dev, uint8_t id,
                                   uint32_t mask, uint32_t *val)
 {
@@ -289,7 +421,6 @@ static int nios_32x32_masked_write(struct bladerf *dev, uint8_t id,
     }
 }
 
-
 int nios_config_read(struct bladerf *dev, uint32_t *val)
 {
     int status = nios_8x32_read(dev, NIOS_PKT_8x32_TARGET_CONTROL, 0, val);
@@ -334,7 +465,8 @@ int nios_get_fpga_version(struct bladerf *dev, struct bladerf_version *ver)
     return status;
 }
 
-int nios_get_timestamp(struct bladerf *dev, bladerf_module module,
+int nios_get_timestamp(struct bladerf *dev,
+                       bladerf_direction dir,
                        uint64_t *timestamp)
 {
     int status;
@@ -342,17 +474,17 @@ int nios_get_timestamp(struct bladerf *dev, bladerf_module module,
     uint8_t addr;
     bool success;
 
-    switch (module) {
-        case BLADERF_MODULE_RX:
+    switch (dir) {
+        case BLADERF_RX:
             addr = NIOS_PKT_8x64_TIMESTAMP_RX;
             break;
 
-        case BLADERF_MODULE_TX:
+        case BLADERF_TX:
             addr = NIOS_PKT_8x64_TIMESTAMP_TX;
             break;
 
         default:
-            log_debug("Invalid module: %d\n", module);
+            log_debug("Invalid direction: %d\n", dir);
             return BLADERF_ERR_INVAL;
     }
 
@@ -366,8 +498,8 @@ int nios_get_timestamp(struct bladerf *dev, bladerf_module module,
     nios_pkt_8x64_resp_unpack(buf, NULL, NULL, NULL, timestamp, &success);
 
     if (success) {
-        log_verbose("%s: Read %s timstamp: 0x%"PRIu64"\n",
-                    __FUNCTION__, module2str(module), timestamp);
+        log_verbose("%s: Read %s timestamp: %" PRIu64 "\n", __FUNCTION__,
+                    direction2str(dir), *timestamp);
         return 0;
     } else {
         log_debug("%s: response packet reported failure.\n", __FUNCTION__);
@@ -380,10 +512,12 @@ int nios_si5338_read(struct bladerf *dev, uint8_t addr, uint8_t *data)
 {
     int status = nios_8x8_read(dev, NIOS_PKT_8x8_TARGET_SI5338, addr, data);
 
+#ifdef ENABLE_LIBBLADERF_NIOS_ACCESS_LOG_VERBOSE
     if (status == 0) {
         log_verbose("%s: Read 0x%02x from addr 0x%02x\n",
                     __FUNCTION__, *data, addr);
     }
+#endif
 
     return status;
 }
@@ -392,10 +526,12 @@ int nios_si5338_write(struct bladerf *dev, uint8_t addr, uint8_t data)
 {
     int status = nios_8x8_write(dev, NIOS_PKT_8x8_TARGET_SI5338, addr, data);
 
+#ifdef ENABLE_LIBBLADERF_NIOS_ACCESS_LOG_VERBOSE
     if (status == 0) {
         log_verbose("%s: Wrote 0x%02x to addr 0x%02x\n",
                     __FUNCTION__, data, addr);
     }
+#endif
 
     return status;
 }
@@ -404,10 +540,12 @@ int nios_lms6_read(struct bladerf *dev, uint8_t addr, uint8_t *data)
 {
     int status = nios_8x8_read(dev, NIOS_PKT_8x8_TARGET_LMS6, addr, data);
 
+#ifdef ENABLE_LIBBLADERF_NIOS_ACCESS_LOG_VERBOSE
     if (status == 0) {
         log_verbose("%s: Read 0x%02x from addr 0x%02x\n",
                     __FUNCTION__, *data, addr);
     }
+#endif
 
     return status;
 }
@@ -416,67 +554,281 @@ int nios_lms6_write(struct bladerf *dev, uint8_t addr, uint8_t data)
 {
     int status = nios_8x8_write(dev, NIOS_PKT_8x8_TARGET_LMS6, addr, data);
 
+#ifdef ENABLE_LIBBLADERF_NIOS_ACCESS_LOG_VERBOSE
     if (status == 0) {
         log_verbose("%s: Wrote 0x%02x to addr 0x%02x\n",
                     __FUNCTION__, data, addr);
     }
+#endif
+
+    return status;
+}
+
+int nios_ina219_read(struct bladerf *dev, uint8_t addr, uint16_t *data)
+{
+    int status;
+
+    status = nios_8x16_read(dev, NIOS_PKT_8x16_TARGET_INA219, addr, data);
+    if (status == 0) {
+        log_verbose("%s: Read 0x%04x from addr 0x%02x\n",
+                    __FUNCTION__, *data, addr);
+    }
+
+    return status;
+}
+
+int nios_ina219_write(struct bladerf *dev, uint8_t addr, uint16_t data)
+{
+    int status;
+
+    status = nios_8x16_write(dev, NIOS_PKT_8x16_TARGET_INA219, addr, data);
+    if (status == 0) {
+        log_verbose("%s: Wrote 0x%04x to addr 0x%02x\n",
+                    __FUNCTION__, data, addr);
+    }
 
     return status;
 }
 
-int nios_vctcxo_trim_dac_write(struct bladerf *dev, uint16_t value)
+#define VERBOSE_OUT_SINGLEBYTE "%s: %s 0x%02x @ addr 0x%04x\n"
+#define VERBOSE_OUT_MULTIBYTE "%s: %s 0x%02x @ addr 0x%04x (%d/%d)\n"
+
+int nios_ad9361_spi_read(struct bladerf *dev, uint16_t cmd, uint64_t *data)
 {
     int status;
 
-    /* Ensure device is in write-through mode */
-    status = nios_8x16_write(dev, NIOS_PKT_8x16_TARGET_VCTCXO_DAC, 0x28, 0);
-    if (status == 0) {
-        /* Write DAC value to channel 0 */
-        status = nios_8x16_write(dev, NIOS_PKT_8x16_TARGET_VCTCXO_DAC,
-                                 0x08, value);
+    status = nios_16x64_read(dev, NIOS_PKT_16x64_TARGET_AD9361, cmd, data);
 
-        log_verbose("%s: Wrote 0x%04x\n", __FUNCTION__, value);
+#ifdef ENABLE_LIBBLADERF_NIOS_ACCESS_LOG_VERBOSE
+    if (log_get_verbosity() == BLADERF_LOG_LEVEL_VERBOSE && status == 0) {
+        size_t bytes = (((cmd >> 12) & 0x7) + 1);
+        size_t addr  = cmd & 0xFFF;
+
+        if (bytes > 1) {
+            size_t i;
+            for (i = 0; i < bytes; ++i) {
+                uint8_t byte = (*data >> (56 - 8 * i)) & 0xFF;
+                log_verbose(VERBOSE_OUT_MULTIBYTE, "ad9361_spi", " MRead", byte,
+                            addr - i, i + 1, bytes);
+            }
+        } else {
+            uint8_t byte = (*data >> 56) & 0xFF;
+            log_verbose(VERBOSE_OUT_SINGLEBYTE, "ad9361_spi", "  Read", byte,
+                        addr);
+        }
     }
+#endif
 
     return status;
 }
 
-int nios_vctcxo_trim_dac_read(struct bladerf *dev, uint16_t *value)
+int nios_ad9361_spi_write(struct bladerf *dev, uint16_t cmd, uint64_t data)
 {
     int status;
 
-    if (!have_cap(dev, BLADERF_CAP_VCTCXO_TRIMDAC_READ)) {
-        *value = 0x0000;
+    status = nios_16x64_write(dev, NIOS_PKT_16x64_TARGET_AD9361, cmd, data);
 
-        log_debug("FPGA %s does not support VCTCXO trimdac readback.\n",
-                  dev->fpga_version.describe);
+#ifdef ENABLE_LIBBLADERF_NIOS_ACCESS_LOG_VERBOSE
+    if (log_get_verbosity() == BLADERF_LOG_LEVEL_VERBOSE && status == 0) {
+        size_t bytes = (((cmd >> 12) & 0x7) + 1) & 0xFF;
+        size_t addr  = cmd & 0xFFF;
 
-        return BLADERF_ERR_UNSUPPORTED;
+        if (bytes > 1) {
+            size_t i;
+            for (i = 0; i < bytes; ++i) {
+                uint8_t byte = (data >> (56 - 8 * i)) & 0xFF;
+                log_verbose(VERBOSE_OUT_MULTIBYTE, "ad9361_spi", "MWrite", byte,
+                            addr - i, i + 1, bytes);
+            }
+        } else {
+            uint8_t byte = (data >> 56) & 0xFF;
+            log_verbose(VERBOSE_OUT_SINGLEBYTE, "ad9361_spi", " Write", byte,
+                        addr);
+        }
     }
+#endif
 
-    status = nios_8x16_read(dev, NIOS_PKT_8x16_TARGET_VCTCXO_DAC, 0x98, value);
-    if (status != 0) {
-        *value = 0;
-    } else {
+    return status;
+}
+
+int nios_adi_axi_read(struct bladerf *dev, uint32_t addr, uint32_t *data)
+{
+    int status;
+
+    status = nios_32x32_read(dev, NIOS_PKT_32x32_TARGET_ADI_AXI, addr, data);
+
+#ifdef ENABLE_LIBBLADERF_NIOS_ACCESS_LOG_VERBOSE
+    if (status == 0) {
+        log_verbose("%s:  Read  0x%08" PRIx32 " from addr 0x%04" PRIx32 "\n",
+                    __FUNCTION__, *data, addr);
+    }
+#endif
+
+    return status;
+}
+
+int nios_adi_axi_write(struct bladerf *dev, uint32_t addr, uint32_t data)
+{
+    int status;
+
+    status = nios_32x32_write(dev, NIOS_PKT_32x32_TARGET_ADI_AXI, addr, data);
+
+#ifdef ENABLE_LIBBLADERF_NIOS_ACCESS_LOG_VERBOSE
+    if (status == 0) {
+        log_verbose("%s: Wrote 0x%08" PRIx32 " to   addr 0x%04" PRIx32 "\n",
+                    __FUNCTION__, data, addr);
+    }
+#endif
+
+    return status;
+}
+
+int nios_rfic_command_read(struct bladerf *dev, uint16_t cmd, uint64_t *data)
+{
+    int status;
+
+    status = nios_16x64_read(dev, NIOS_PKT_16x64_TARGET_RFIC, cmd, data);
+
+#ifdef ENABLE_LIBBLADERF_NIOS_ACCESS_LOG_VERBOSE
+    if (status == 0) {
+        log_verbose("%s: Read 0x%04x 0x%08x\n", __FUNCTION__, cmd, *data);
+    }
+#endif
+
+    return status;
+}
+
+int nios_rfic_command_write(struct bladerf *dev, uint16_t cmd, uint64_t data)
+{
+    int status;
+
+    status = nios_16x64_write(dev, NIOS_PKT_16x64_TARGET_RFIC, cmd, data);
+
+#ifdef ENABLE_LIBBLADERF_NIOS_ACCESS_LOG_VERBOSE
+    if (status == 0) {
+        log_verbose("%s: Write 0x%04x 0x%08x\n", __FUNCTION__, cmd, data);
+    }
+#endif
+
+    return status;
+}
+
+int nios_rffe_control_read(struct bladerf *dev, uint32_t *value)
+{
+    int status;
+
+    status = nios_8x32_read(dev, NIOS_PKT_8x32_TARGET_RFFE_CSR, 0, value);
+
+#ifdef ENABLE_LIBBLADERF_NIOS_ACCESS_LOG_VERBOSE
+    if (status == 0) {
+        log_verbose("%s: Read 0x%08x\n", __FUNCTION__, *value);
+    }
+#endif
+
+    return status;
+}
+
+int nios_rffe_control_write(struct bladerf *dev, uint32_t value)
+{
+    int status;
+
+    status = nios_8x32_write(dev, NIOS_PKT_8x32_TARGET_RFFE_CSR, 0, value);
+
+#ifdef ENABLE_LIBBLADERF_NIOS_ACCESS_LOG_VERBOSE
+    if (status == 0) {
+        log_verbose("%s: Wrote 0x%08x\n", __FUNCTION__, value);
+    }
+#endif
+
+    return status;
+}
+
+int nios_rffe_fastlock_save(struct bladerf *dev, bool is_tx,
+                            uint8_t rffe_profile, uint16_t nios_profile)
+{
+    int status;
+    uint8_t  addr;
+    uint32_t data = 0;
+
+    addr = is_tx ? 1 : 0;
+    data = (rffe_profile << 16) | nios_profile;
+
+    status = nios_8x32_write(dev, NIOS_PKT_8x32_TARGET_FASTLOCK, addr, data);
+
+#ifdef ENABLE_LIBBLADERF_NIOS_ACCESS_LOG_VERBOSE
+    if (status == 0) {
+        log_verbose("%s: Wrote 0x%08x\n", __FUNCTION__, data);
+    }
+#endif
+
+    return status;
+}
+
+int nios_ad56x1_vctcxo_trim_dac_read(struct bladerf *dev, uint16_t *value)
+{
+    int status;
+
+    status = nios_8x16_read(dev, NIOS_PKT_8x16_TARGET_AD56X1_DAC, 0, value);
+    if (status == 0) {
         log_verbose("%s: Read 0x%04x\n", __FUNCTION__, *value);
     }
 
     return status;
 }
 
+int nios_ad56x1_vctcxo_trim_dac_write(struct bladerf *dev, uint16_t value)
+{
+    int status;
+
+    status = nios_8x16_write(dev, NIOS_PKT_8x16_TARGET_AD56X1_DAC, 0, value);
+    if (status == 0) {
+        log_verbose("%s: Wrote 0x%04x\n", __FUNCTION__, value);
+    }
+
+    return status;
+}
+
+int nios_adf400x_read(struct bladerf *dev, uint8_t addr, uint32_t *data)
+{
+    int status;
+
+    status = nios_8x32_read(dev, NIOS_PKT_8x32_TARGET_ADF400X, addr, data);
+    if (status == 0) {
+        log_verbose("%s: Read 0x%08x from addr 0x%02x\n", __FUNCTION__, *data, addr);
+    }
+
+    return status;
+}
+
+int nios_adf400x_write(struct bladerf *dev, uint8_t addr, uint32_t data)
+{
+    int status;
+
+    data &= ~(0x3);
+
+    status = nios_8x32_write(dev, NIOS_PKT_8x32_TARGET_ADF400X, 0, data | (addr & 0x3));
+    if (status == 0) {
+        log_verbose("%s: Wrote 0x%08x to addr 0x%02x\n", __FUNCTION__, data, addr);
+    }
+
+    return status;
+}
+
+int nios_vctcxo_trim_dac_write(struct bladerf *dev, uint8_t addr, uint16_t value)
+{
+    return nios_8x16_write(dev, NIOS_PKT_8x16_TARGET_VCTCXO_DAC, addr, value);
+}
+
+int nios_vctcxo_trim_dac_read(struct bladerf *dev, uint8_t addr, uint16_t *value)
+{
+    return nios_8x16_read(dev, NIOS_PKT_8x16_TARGET_VCTCXO_DAC, addr, value);
+}
+
 int nios_set_vctcxo_tamer_mode(struct bladerf *dev,
                                bladerf_vctcxo_tamer_mode mode)
 {
     int status;
 
-    if (!have_cap(dev, BLADERF_CAP_VCTCXO_TAMING_MODE)) {
-        log_debug("FPGA %s does not support VCTCXO taming via an input source\n",
-                  dev->fpga_version.describe);
-
-        return BLADERF_ERR_UNSUPPORTED;
-    }
-
-
     status = nios_8x8_write(dev,
                             NIOS_PKT_8x8_TARGET_VCTCXO_TAMER, 0xff,
                             (uint8_t) mode);
@@ -496,13 +848,6 @@ int nios_get_vctcxo_tamer_mode(struct bladerf *dev,
 
     *mode = BLADERF_VCTCXO_TAMER_INVALID;
 
-    if (!have_cap(dev, BLADERF_CAP_VCTCXO_TAMING_MODE)) {
-        log_debug("FPGA %s does not support VCTCXO taming via an input source\n",
-                  dev->fpga_version.describe);
-
-        return BLADERF_ERR_UNSUPPORTED;
-    }
-
     status = nios_8x8_read(dev, NIOS_PKT_8x8_TARGET_VCTCXO_TAMER, 0xff, &tmp);
     if (status == 0) {
         log_verbose("%s: Read mode=0x%02x\n", __FUNCTION__, tmp);
@@ -523,129 +868,151 @@ int nios_get_vctcxo_tamer_mode(struct bladerf *dev,
 }
 
 
-int nios_get_iq_gain_correction(struct bladerf *dev, bladerf_module module,
+int nios_get_iq_gain_correction(struct bladerf *dev, bladerf_channel ch,
                                 int16_t *value)
 {
     int status = BLADERF_ERR_INVAL;
     uint16_t tmp = 0;
 
-    switch (module) {
-        case BLADERF_MODULE_RX:
+    switch (ch) {
+        case BLADERF_CHANNEL_RX(0):
             status = nios_8x16_read(dev, NIOS_PKT_8x16_TARGET_IQ_CORR,
                                     NIOS_PKT_8x16_ADDR_IQ_CORR_RX_GAIN, &tmp);
             break;
 
-        case BLADERF_MODULE_TX:
+        case BLADERF_CHANNEL_TX(0):
             status = nios_8x16_read(dev, NIOS_PKT_8x16_TARGET_IQ_CORR,
                                     NIOS_PKT_8x16_ADDR_IQ_CORR_TX_GAIN, &tmp);
             break;
 
         default:
-            log_debug("Invalid module: %d\n", module);
+            log_debug("Invalid channel: 0x%x\n", ch);
     }
 
     *value = (int16_t) tmp;
 
     if (status == 0) {
         log_verbose("%s: Read %s %d\n",
-                    __FUNCTION__, module2str(module), *value);
+                    __FUNCTION__, channel2str(ch), *value);
     }
 
     return status;
 }
 
-int nios_get_iq_phase_correction(struct bladerf *dev, bladerf_module module,
+int nios_get_iq_phase_correction(struct bladerf *dev, bladerf_channel ch,
                                  int16_t *value)
 {
     int status = BLADERF_ERR_INVAL;
     uint16_t tmp = 0;
 
-    switch (module) {
-        case BLADERF_MODULE_RX:
+    switch (ch) {
+        case BLADERF_CHANNEL_RX(0):
             status = nios_8x16_read(dev, NIOS_PKT_8x16_TARGET_IQ_CORR,
                                     NIOS_PKT_8x16_ADDR_IQ_CORR_RX_PHASE, &tmp);
             break;
 
-        case BLADERF_MODULE_TX:
+        case BLADERF_CHANNEL_TX(0):
             status = nios_8x16_read(dev, NIOS_PKT_8x16_TARGET_IQ_CORR,
                                     NIOS_PKT_8x16_ADDR_IQ_CORR_TX_PHASE, &tmp);
             break;
 
         default:
-            log_debug("Invalid module: %d\n", module);
+            log_debug("Invalid channel: 0x%x\n", ch);
     }
 
     *value = (int16_t) tmp;
 
     if (status == 0) {
         log_verbose("%s: Read %s %d\n",
-                    __FUNCTION__, module2str(module), *value);
+                    __FUNCTION__, channel2str(ch), *value);
     }
 
     return status;
 }
 
-int nios_set_iq_gain_correction(struct bladerf *dev, bladerf_module module,
+int nios_set_iq_gain_correction(struct bladerf *dev, bladerf_channel ch,
                                 int16_t value)
 {
     int status = BLADERF_ERR_INVAL;
 
-    switch (module) {
-        case BLADERF_MODULE_RX:
+    switch (ch) {
+        case BLADERF_CHANNEL_RX(0):
             log_verbose("Setting RX IQ Correction gain: %d\n", value);
             status = nios_8x16_write(dev, NIOS_PKT_8x16_TARGET_IQ_CORR,
                                      NIOS_PKT_8x16_ADDR_IQ_CORR_RX_GAIN, value);
             break;
 
-        case BLADERF_MODULE_TX:
+        case BLADERF_CHANNEL_TX(0):
             log_verbose("Setting TX IQ Correction gain: %d\n", value);
             status = nios_8x16_write(dev, NIOS_PKT_8x16_TARGET_IQ_CORR,
                                      NIOS_PKT_8x16_ADDR_IQ_CORR_TX_GAIN, value);
             break;
 
         default:
-            log_debug("Invalid module: %d\n", module);
+            log_debug("Invalid channel: 0x%x\n", ch);
     }
 
     if (status == 0) {
         log_verbose("%s: Wrote %s %d\n",
-                    __FUNCTION__, module2str(module), value);
+                    __FUNCTION__, channel2str(ch), value);
     }
 
     return status;
 }
 
-int nios_set_iq_phase_correction(struct bladerf *dev, bladerf_module module,
+int nios_set_iq_phase_correction(struct bladerf *dev, bladerf_channel ch,
                                  int16_t value)
 {
     int status = BLADERF_ERR_INVAL;
 
-    switch (module) {
-        case BLADERF_MODULE_RX:
+    switch (ch) {
+        case BLADERF_CHANNEL_RX(0):
             log_verbose("Setting RX IQ Correction phase: %d\n", value);
             status = nios_8x16_write(dev, NIOS_PKT_8x16_TARGET_IQ_CORR,
                                      NIOS_PKT_8x16_ADDR_IQ_CORR_RX_PHASE, value);
             break;
 
-        case BLADERF_MODULE_TX:
+        case BLADERF_CHANNEL_TX(0):
             log_verbose("Setting TX IQ Correction phase: %d\n", value);
             status = nios_8x16_write(dev, NIOS_PKT_8x16_TARGET_IQ_CORR,
                                      NIOS_PKT_8x16_ADDR_IQ_CORR_TX_PHASE, value);
             break;
 
         default:
-            log_debug("Invalid module: %d\n", module);
+            log_debug("Invalid channel: 0x%x\n", ch);
     }
 
     if (status == 0) {
         log_verbose("%s: Wrote %s %d\n",
-                    __FUNCTION__, module2str(module), value);
+                    __FUNCTION__, channel2str(ch), value);
     }
 
 
     return status;
 }
 
+int nios_set_agc_dc_correction(struct bladerf *dev, int16_t q_max, int16_t i_max,
+                               int16_t q_mid, int16_t i_mid,
+                               int16_t q_low, int16_t i_low)
+{
+    int status;
+
+    status = nios_8x16_write(dev, NIOS_PKT_8x16_TARGET_AGC_CORR, NIOS_PKT_8x16_ADDR_AGC_DC_Q_MAX, q_max);
+
+    if (!status)
+        status = nios_8x16_write(dev, NIOS_PKT_8x16_TARGET_AGC_CORR, NIOS_PKT_8x16_ADDR_AGC_DC_I_MAX, i_max);
+    if (!status)
+        status = nios_8x16_write(dev, NIOS_PKT_8x16_TARGET_AGC_CORR, NIOS_PKT_8x16_ADDR_AGC_DC_Q_MID, q_mid);
+    if (!status)
+        status = nios_8x16_write(dev, NIOS_PKT_8x16_TARGET_AGC_CORR, NIOS_PKT_8x16_ADDR_AGC_DC_I_MID, i_mid);
+    if (!status)
+        status = nios_8x16_write(dev, NIOS_PKT_8x16_TARGET_AGC_CORR, NIOS_PKT_8x16_ADDR_AGC_DC_Q_MIN, q_low);
+    if (!status)
+        status = nios_8x16_write(dev, NIOS_PKT_8x16_TARGET_AGC_CORR, NIOS_PKT_8x16_ADDR_AGC_DC_I_MIN, i_low);
+
+    return status;
+}
+
 int nios_xb200_synth_write(struct bladerf *dev, uint32_t value)
 {
     int status = nios_8x32_write(dev, NIOS_PKT_8x32_TARGET_ADF4351, 0, value);
@@ -708,10 +1075,10 @@ int nios_expansion_gpio_dir_write(struct bladerf *dev,
     return status;
 }
 
-int nios_retune(struct bladerf *dev, bladerf_module module,
+int nios_retune(struct bladerf *dev, bladerf_channel ch,
                 uint64_t timestamp, uint16_t nint, uint32_t nfrac,
                 uint8_t freqsel, uint8_t vcocap, bool low_band,
-                bool quick_tune)
+                uint8_t xb_gpio, bool quick_tune)
 {
     int status;
     uint8_t buf[NIOS_PKT_LEN];
@@ -720,16 +1087,17 @@ int nios_retune(struct bladerf *dev, bladerf_module module,
     uint64_t duration;
 
     if (timestamp == NIOS_PKT_RETUNE_CLEAR_QUEUE) {
-        log_verbose("Clearing %s retune queue.\n", module2str(module));
+        log_verbose("Clearing %s retune queue.\n", channel2str(ch));
     } else {
-        log_verbose("%s: module=%s timestamp=%"PRIu64" nint=%u nfrac=%u\n\t\t\t\t"
+        log_verbose("%s: channel=%s timestamp=%"PRIu64" nint=%u nfrac=%u\n\t\t\t\t"
                     "freqsel=0x%02x vcocap=0x%02x low_band=%d quick_tune=%d\n",
-                    __FUNCTION__, module2str(module), timestamp, nint, nfrac,
+                    __FUNCTION__, channel2str(ch), timestamp, nint, nfrac,
                     freqsel, vcocap, low_band, quick_tune);
     }
 
-    nios_pkt_retune_pack(buf, module, timestamp,
-                         nint, nfrac, freqsel, vcocap, low_band, quick_tune);
+    nios_pkt_retune_pack(buf, ch, timestamp,
+                         nint, nfrac, freqsel, vcocap, low_band,
+                         xb_gpio, quick_tune);
 
     status = nios_access(dev, buf);
     if (status != 0) {
@@ -740,10 +1108,10 @@ int nios_retune(struct bladerf *dev, bladerf_module module,
 
     if (resp_flags & NIOS_PKT_RETUNERESP_FLAG_TSVTUNE_VALID) {
         log_verbose("%s retune operation: vcocap=%u, duration=%"PRIu64"\n",
-                    module2str(module), vcocap, duration);
+                    channel2str(ch), vcocap, duration);
     } else {
         log_verbose("%s operation duration: %"PRIu64"\n",
-                    module2str(module), duration);
+                    channel2str(ch), duration);
     }
 
     if ((resp_flags & NIOS_PKT_RETUNERESP_FLAG_SUCCESS) == 0) {
@@ -760,29 +1128,83 @@ int nios_retune(struct bladerf *dev, bladerf_module module,
     return status;
 }
 
-int nios_read_trigger(struct bladerf *dev, bladerf_module module,
+int nios_retune2(struct bladerf *dev, bladerf_channel ch,
+                 uint64_t timestamp, uint16_t nios_profile,
+                 uint8_t rffe_profile, uint8_t port,
+                 uint8_t spdt)
+{
+    int status;
+    uint8_t buf[NIOS_PKT_LEN];
+
+    uint8_t resp_flags;
+    uint64_t duration;
+
+    if (timestamp == NIOS_PKT_RETUNE2_CLEAR_QUEUE) {
+        log_verbose("Clearing %s retune queue.\n", channel2str(ch));
+    } else {
+        log_verbose("%s: channel=%s timestamp=%"PRIu64" nios_profile=%u "
+                    "rffe_profile=%u\n\t\t\t\tport=0x%02x spdt=0x%02x\n",
+                    __FUNCTION__, channel2str(ch), timestamp, nios_profile,
+                    rffe_profile, port, spdt);
+    }
+
+    nios_pkt_retune2_pack(buf, ch, timestamp, nios_profile, rffe_profile,
+                          port, spdt);
+
+    status = nios_access(dev, buf);
+    if (status != 0) {
+        return status;
+    }
+
+    nios_pkt_retune2_resp_unpack(buf, &duration, &resp_flags);
+
+    if (resp_flags & NIOS_PKT_RETUNE2_RESP_FLAG_TSVTUNE_VALID) {
+        log_verbose("%s retune operation: duration=%"PRIu64"\n",
+                    channel2str(ch), duration);
+    } else {
+        log_verbose("%s operation duration: %"PRIu64"\n",
+                    channel2str(ch), duration);
+    }
+
+    if ((resp_flags & NIOS_PKT_RETUNE2_RESP_FLAG_SUCCESS) == 0) {
+        if (timestamp == BLADERF_RETUNE_NOW) {
+            log_debug("FPGA tuning reported failure.\n");
+            status = BLADERF_ERR_UNEXPECTED;
+        } else {
+            log_debug("The FPGA's retune queue is full. Try again after "
+                      "a previous request has completed.\n");
+            status = BLADERF_ERR_QUEUE_FULL;
+        }
+    }
+
+    return status;
+}
+
+int nios_read_trigger(struct bladerf *dev, bladerf_channel ch,
                       bladerf_trigger_signal trigger, uint8_t *value)
 {
     int status;
     uint8_t nios_id;
 
-    switch (module) {
-        case BLADERF_MODULE_TX:
+    switch (ch) {
+        case BLADERF_CHANNEL_TX(0):
             nios_id = NIOS_PKT_8x8_TX_TRIGGER_CTL;
             break;
 
-        case BLADERF_MODULE_RX:
+        case BLADERF_CHANNEL_RX(0):
             nios_id = NIOS_PKT_8x8_RX_TRIGGER_CTL;
             break;
 
         default:
-            log_debug("Invalid module: %d\n", module);
+            log_debug("Invalid channel: 0x%x\n", ch);
             return BLADERF_ERR_INVAL;
     }
 
     /* Only 1 external trigger is currently supported */
     switch (trigger) {
         case BLADERF_TRIGGER_J71_4:
+        case BLADERF_TRIGGER_J51_1:
+        case BLADERF_TRIGGER_MINI_EXP_1:
             break;
 
         default:
@@ -792,35 +1214,37 @@ int nios_read_trigger(struct bladerf *dev, bladerf_module module,
 
     status = nios_8x8_read(dev, nios_id, 0, value);
     if (status == 0) {
-        log_verbose("%s trigger read value 0x%02x\n", module2str(module), *value);
+        log_verbose("%s trigger read value 0x%02x\n", channel2str(ch), *value);
     }
 
     return status;
 }
 
-int nios_write_trigger(struct bladerf *dev, bladerf_module module,
+int nios_write_trigger(struct bladerf *dev, bladerf_channel ch,
                        bladerf_trigger_signal trigger, uint8_t value)
 {
     int status;
     uint8_t nios_id;
 
-    switch (module) {
-        case BLADERF_MODULE_TX:
+    switch (ch) {
+        case BLADERF_CHANNEL_TX(0):
             nios_id = NIOS_PKT_8x8_TX_TRIGGER_CTL;
             break;
 
-        case BLADERF_MODULE_RX:
+        case BLADERF_CHANNEL_RX(0):
             nios_id = NIOS_PKT_8x8_RX_TRIGGER_CTL;
             break;
 
         default:
-            log_debug("Invalid module: %d\n", module);
+            log_debug("Invalid channel: 0x%x\n", ch);
             return BLADERF_ERR_INVAL;
     }
 
     /* Only 1 external trigger is currently supported */
     switch (trigger) {
         case BLADERF_TRIGGER_J71_4:
+        case BLADERF_TRIGGER_J51_1:
+        case BLADERF_TRIGGER_MINI_EXP_1:
             break;
 
         default:
@@ -830,7 +1254,7 @@ int nios_write_trigger(struct bladerf *dev, bladerf_module module,
 
     status = nios_8x8_write(dev, nios_id, 0, value);
     if (status == 0) {
-        log_verbose("%s trigger write value 0x%02x\n", module2str(module), value);
+        log_verbose("%s trigger write value 0x%02x\n", channel2str(ch), value);
     }
 
     return status;
